

CREATE OR REPLACE FUNCTION spJnl_New (
	_JnlNo		jnl.jnlno%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spJnl_New(_JnlNo, TRUE, TRUE);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnl_New(
	_JnlNo		VARCHAR(15),
	_CheckLockDate	BOOLEAN,
	_CheckPer	BOOLEAN
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_AccNo			VARCHAR(10);
	_JnlDate		DATE;
	_amount			NUMERIC(19,4);
	_amountD		NUMERIC(19,4);
	_amountK		NUMERIC(19,4);
	_Count			INT;
	_DK			VARCHAR(1);
	_DefDK			VARCHAR(1);
	_AccType2 		VARCHAR(5);
	_CashID 		VARCHAR(15);
	_Auto			BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_JnlNote		VARCHAR(255);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_tmpNote		VARCHAR(255);
	_RefType		VARCHAR;
	_RefNo			VARCHAR;
	_LockDate		DATE;
	_PerStart		DATE;

	tAccNo			VARCHAR;
	tJnlNo			VARCHAR;
	tJnlDNo			SMALLINT;

	cAcc			REFCURSOR;
	ctmp			REFCURSOR;
	cJnlD			REFCURSOR;

	_JnlDNo			SMALLINT;
	_selisih		DECIMAL(19,4);
	_JnlDAmt		DECIMAL(19,4);
	_JnlDAmt_Rounded	DECIMAL(19,4);
	_JnlDAmt_selisih	DECIMAL(19,4);

	_debet_found 		BOOLEAN;
	_kredit_found 		BOOLEAN;

	_jnldcount		INT;
	_ronde			INT;

	_SumDebit		NUMERIC(19,4);
	_SumCredit		NUMERIC(19,4);
BEGIN
	-- Dont process posting, for debugging purpose, to see what is the jurnal details amounts before gets actually posted

	--_retval:=(0, 'OK');
	--RETURN NEXT _retval;
	--RETURN;

	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID, _RefType, _RefNo
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID, reftype, refno 
	FROM jnl 
	WHERE JnlNo=_JnlNo;
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Jurnal Voucher#'||COALESCE(_JnlNo, '<kosong>')||' tidak ditemukan ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _jnldcount count(*) FROM jnld WHERE jnlNo=_JnlNo;

	IF (_jnldcount IS NULL OR _jnldcount=0) THEN
		DELETE FROM jnl WHERE JnlNo=_JnlNo;
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_ronde:=0;
	LOOP
		_ronde:=_ronde+1;

		SELECT INTO _amountD SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
		_debet_found:=FOUND;
		SELECT INTO _amountK SUM(credit) FROM jnld WHERE JnlNo=_JnlNo;
		_kredit_found:=FOUND;

		IF (_debet_found=FALSE AND _kredit_found=FALSE) THEN
			RAISE EXCEPTION 'Tidak ada transaksi di DEBET maupun KREDIT (spJnl_New)';
		END IF;

		_amountD:=COALESCE(_amountD, 0);
		_amountK:=COALESCE(_amountK, 0);

		_selisih:=_amountD-_amountK;

		EXIT WHEN (_selisih=0);

		IF (_ronde>1 AND _selisih<>0) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Referensi('||COALESCE(_RefType, '<kosong>')||')#'||COALESCE(_RefNo, '<kosong>')||E'\n'||COALESCE(_JnlNote, '<kosong>')||E'\n'||'Jumlah Debet Kredit Tidak Sama!(spJnl_New) '||_amountD||' -VS- '||_amountK;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_selisih<>0) THEN
			OPEN cJnlD FOR 
				SELECT DK, JnlDNo, JnlDAmt FROM JnlD WHERE JnlNo=_JnlNo;
			LOOP
				FETCH cJnlD INTO _DK, _JnlDNo, _JnlDAmt;
				EXIT WHEN NOT FOUND;

				_JnlDAmt_Rounded:=ROUND(_JnlDAmt, 4);
				_JnlDAmt_selisih:=_JnlDAmt-_JnlDAmt_Rounded;
			
				IF (_JnlDAmt_selisih<>0) THEN
					IF (_DK='D') THEN
						UPDATE JnlD SET Debit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					ELSIF (_DK='K') THEN
						UPDATE JnlD SET Credit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					END IF;
				END IF;
			END LOOP;
			CLOSE cJnlD;
		END IF;
	END LOOP;


/** fail safe kalau kelewatan di loop tsb di atas */
	SELECT INTO _SumDebit, _SumCredit SUM(d.debit) AS debit, SUM(d.credit) 
	FROM jnld d
	WHERE jnlno=_JnlNo;
	IF FOUND THEN
		IF (_SumDebit<>_SumCredit) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Fail: Referensi('||COALESCE(_RefType, '<kosong>')||')#'||COALESCE(_RefNo, '<kosong>')||E'\n'||COALESCE(_JnlNote, '<kosong>')||E'\n'||'Jumlah Debet Kredit Tidak Sama!(spJnl_New) '||_SumDebit||' -VS- '||_SumCredit;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID
	FROM jnl 
	WHERE JnlNo=_JnlNo;

	IF (_CheckPer=TRUE) THEN
		SELECT INTO _PerStart perstart from per where active=true;
		IF(_PerStart>_JnlDate) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF (_CheckLockDate=TRUE) THEN
		SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
		IF(_LockDate is not null AND _LockDate>_JnlDate) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	SELECT INTO _amount SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
	UPDATE jnl SET jnlamt=_amount WHERE JnlNo=_JnlNo;

	_tmpNote:=_JnlNote;

	INSERT INTO lgr(
		accno, jnlno, jnldno, lgrdate, lgramt, lgrnote, 
		PerID, BranchID, DeptID, PrjID, LgrAmtForex, CrcID, ExcRate, FisRate
		)
	SELECT 
		d.accno, d.jnlno, jnldno, _JnlDate,
		CASE WHEN a.accdk=d.dk THEN jnldamt ELSE jnldamt*-1 END,
		COALESCE(jnlnote,' ')||' '||COALESCE(jnldnote,' '),
		j.PerID,  j.BranchID, d.DeptID, d.PrjID, JnlDAmtForex, j.CrcID, j.ExcRate, j.FisRate
	FROM jnld d
	JOIN jnl j on j.jnlno=d.jnlno 
	JOIN acc a on d.accno=a.accno
	WHERE d.JnlNo=_JnlNo;

	OPEN cAcc FOR
		SELECT 
			acctype2.acctype2, jnld.accno, acc.AccDK, dk, jnldamt, branchid, deptid, prjid 
		FROM jnld
		JOIN jnl ON jnl.jnlno=jnld.jnlno 
		JOIN acc ON jnld.accno=acc.accno 
		LEFT JOIN acctype2 ON acc.acctype2=acctype2.acctype2
		WHERE jnld.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccType2, _AccNo, _DefDK, _DK, _amount, _BranchID, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF (_DefDK<>_DK) THEN	
			_amount:=_amount*-1;
		END IF;

		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_JnlDate,_amount, _BranchID, _DeptID, _PrjID);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cAcc;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


